@@ -13,7 +13,7 @@ import logging
 import os
 import pickle
 from datetime import datetime
-from typing import Dict, List, Union
+from typing import Union
 
 import pandas as pd
 
@@ -23,11 +23,33 @@ from curifactory import hashing, utils
 class Lazy:
     """A class to indicate a stage output as a lazy-cache object - curifactory will
     attempt to keep this out of memory as much as possible, immediately caching and deleting,
-    and loading back into memeory only when needed."""
+    and loading back into memeory only when needed.
 
-    def __init__(self, name: str):
+    This object is used by "wrapping" a stage output string with the class.
+
+    Example:
+        .. code-block:: python
+
+            @stage(inputs=None, outputs=["small_output", cf.Lazy("large_output")], cachers=[PickleCacher]*2)
+            def some_stage(record: Record):
+                ...
+
+    Args:
+        name (str): the name of the output to put into state.
+        resolve (bool): Whether this lazy object should automatically reload the initial
+            object when accessed from state. By default this is ``True`` - when a stage specifies
+            the string name as an input and this object is requested from the record state, it
+            loads and passes in the originally stored object. If set to ``False``, the stage
+            input will instead be populated with the lazy object itself, giving the inner stage
+            code direct access to the cacher. This is useful if you need to keep objects out of
+            memory and just want to refer to the cacher path (e.g. to send this path along
+            to an external CLI/script.)
+    """
+
+    def __init__(self, name: str, resolve: bool = True):
         self.name = name
         self.cacher: Cacheable = None
+        self.resolve = resolve
 
     def load(self):
         return self.cacher.load()
@@ -116,11 +138,11 @@ class Cacheable:
         how artifact metadata is collected."""
         self.track = track
         """Whether to store the artifact this cacher is used with in the run folder on store-full runs or not."""
-        self.cache_paths: List[str] = []
+        self.cache_paths: list[str] = []
         """The running list of paths this cacher is using, as appended by ``get_path``."""
-        self.metadata: Dict = None
+        self.metadata: dict = None
         """Metadata about the artifact cached with this cacheable."""
-        self.extra_metadata: Dict = {}
+        self.extra_metadata: dict = {}
         """``collect_metadata`` uses but does not overwrite this, placing into the `extra` key
         in the actual metadata. This can be used by the cacher's save function to store additional
         information that would then be available if the 'load' function calls ``load_metadata()``."""
@@ -273,7 +295,7 @@ class Cacheable:
         with open(metadata_path, "w") as outfile:
             json.dump(self.metadata, outfile, indent=2, default=str)
 
-    def load_metadata(self) -> Dict:
+    def load_metadata(self) -> dict:
         metadata_path = self.get_path("_metadata.json")
         if os.path.exists(metadata_path):
             with open(metadata_path) as infile:
@@ -407,8 +429,8 @@ class PandasJsonCacher(Cacheable):
     def __init__(
         self,
         path_override: str = None,
-        to_json_args: Dict = dict(double_precision=15),
-        read_json_args: Dict = dict(),
+        to_json_args: dict = dict(double_precision=15),
+        read_json_args: dict = dict(),
         **kwargs
     ):
         self.read_json_args = read_json_args
@@ -437,8 +459,8 @@ class PandasCsvCacher(Cacheable):
     def __init__(
         self,
         path_override: str = None,
-        to_csv_args: Dict = dict(),
-        read_csv_args: Dict = dict(index_col=0),
+        to_csv_args: dict = dict(),
+        read_csv_args: dict = dict(index_col=0),
         **kwargs
     ):
         self.read_csv_args = read_csv_args
@@ -514,12 +536,12 @@ class FileReferenceCacher(Cacheable):
 
         return True
 
-    def load(self) -> Union[List[str], str]:
+    def load(self) -> Union[list[str], str]:
         with open(self.get_path()) as infile:
             files = json.load(infile)
         return files
 
-    def save(self, files: Union[List[str], str]) -> str:
+    def save(self, files: Union[list[str], str]) -> str:
         path = self.get_path()
         with open(path, "w") as outfile:
             json.dump(files, outfile, indent=4)
